import puppeteer from "puppeteer";
import { TemplateHelper } from "./templateHelper";
import { PermissionModule } from "../type/permissionTypes";
import { FileSystemHelper } from "./fileSystemHelper";
import { PDFDocument } from "pdf-lib";

export abstract class PdfExport {
  static async renderFile({
    template,
    title = "pdf-export Mitgliederverwaltung",
    filename = null,
    data = {},
    saveToDisk = true,
    margins = { top: "15mm", bottom: "15mm" },
    folder = "",
  }: {
    template: PermissionModule;
    title?: string;
    filename?: string;
    data?: any;
    saveToDisk?: boolean;
    margins?: { top: string; bottom: string };
    folder?: string;
  }) {
    if (folder != "") FileSystemHelper.createFolder(folder);

    const { header, footer, body } = await TemplateHelper.renderFileForModule({
      module: template,
      bodyData: data,
      title: title,
    });

    const browser = await puppeteer.launch({
      headless: true,
      args: ["--no-sandbox", "--disable-gpu", "--disable-setuid-sandbox"],
    });
    const page = await browser.newPage();
    await page.setContent(body, { waitUntil: "domcontentloaded" });

    const exportPath = FileSystemHelper.formatPath(process.cwd(), "export", folder, `${filename}.pdf`);

    let pdf = await page.pdf({
      ...(saveToDisk ? { path: exportPath } : {}),
      format: "A4",
      printBackground: false,
      margin: {
        top: margins.top,
        bottom: margins.bottom,
        left: "10mm",
        right: "10mm",
      },
      displayHeaderFooter: true,
      headerTemplate: header,
      footerTemplate: footer,
    });

    await browser.close();

    return pdf;
  }

  static async sqashToSingleFile(inputFolder: string, outputFile: string, outputFolder: string = "") {
    if (outputFolder != "") FileSystemHelper.createFolder(outputFolder);

    let pdfFilePaths = FileSystemHelper.getFilesInDirectory(inputFolder, ".pdf");

    const mergedPdf = await PDFDocument.create();

    for (const pdfPath of pdfFilePaths) {
      const pdfBytes = FileSystemHelper.readFile(pdfPath);
      const pdf = await PDFDocument.load(pdfBytes);
      const copiedPages = await mergedPdf.copyPages(pdf, pdf.getPageIndices());
      copiedPages.forEach((page) => mergedPdf.addPage(page));
    }

    const mergedPdfBytes = await mergedPdf.save();

    const exportPath = FileSystemHelper.formatPath(process.cwd(), "export", outputFolder, `${outputFile}.pdf`);

    FileSystemHelper.writeFile(exportPath, mergedPdfBytes);
  }
}
